#include <stdio.h>
#include <sys/utsname.h>
#include <string.h>
#include <math.h>
#include "speed.h"

#ifndef PERF_DOT_CONF
#define PERF_DOT_CONF "/etc/pperf.conf"
#endif

int machine ( char *mach, float *mhz, int *events, int fly )
{
  struct utsname test;
  char buf[256];
  char *p1, *p2;
  float bogo, diff;
  FILE *fp;
  
  *mhz = 0;

  uname(&test);

  strcpy(mach,test.machine);

  if ( strcmp(mach,"i586") && strcmp(mach,"i686") )
    {
      fprintf(stderr,
	      "error: Fatal error.  Architecture %s not supported.\n",mach);
      fprintf(stderr,
	      "\tThis software only works on i586 and i686 machines.\n");
      exit(0);
    }

  if (!fly) /* Compute CPU speed on the fly */
    {
      *mhz = speed();
    }
  else /* Read CPU_SPEED in PERF_DOT_CONF */
    {
      if ( (fp = fopen(PERF_DOT_CONF,"r")) == NULL )
	{
	  *mhz = speed();
	  fprintf(stderr,
		  "warning: Cannot open %s.\n",PERF_DOT_CONF);
	  fprintf(stderr,
		  "\tSetting CPU_SPEED to computed value %.2f mhz.\n",
		  *mhz);
	  fprintf(stderr,
		  "\tRates reported will likely be incorrect.\n");
	  fprintf(stderr,
		  "\tTo fix this problem set the variable CPU_SPEED\n");
	  fprintf(stderr,
		  "\tin %s as described in the manual page.\n",
		  PERF_DOT_CONF);
	}
      else
	{      
	  while ( fgets(buf,sizeof(buf),fp) != NULL )
	    {
	      /* Break the line up at '=' into two parts. */
	      p1 = strtok(buf,"=");
	      p2 = strtok(NULL,"=");
	      /* Get rid of space if it's there */
	      p1 = strtok(p1," ");
	      p2 = strtok(p2," ");
	      if (strcmp(p1,"CPU_SPEED") == 0)
		{
		  *mhz = (float) atoi(p2);
		}
	    }
      
	  if (*mhz == 0)
	    {
	      *mhz = speed();
	      fprintf(stderr,
		      "warning: Could not find CPU_SPEED= in %s\n",
		      PERF_DOT_CONF);
	      fprintf(stderr,
		      "\tSetting CPU_SPEED to computed value: %.2fmhz.\n",
		      *mhz);
	      fprintf(stderr,
		      "\tRates reported will likely be incorrect.\n");
	      fprintf(stderr,
		      "\tTo fix problem set the variable CPU_SPEED\n");
	      fprintf(stderr,"       in %s as described in the manual page.\n",
		      PERF_DOT_CONF);
	    }
	}
      close(fp);
    }

  if (!fly && !strcmp(mach,"i686"))
    {
      bogo = 0.0;
      if ( (fp = fopen("/proc/cpuinfo","r")) == NULL )
	{
	  fprintf(stderr,"warning: Cannot open /proc/cpuinfo.\n");
	  fprintf(stderr,"\tUsing %.2f *mhz.\n",*mhz);
	}
      else
	{
	  while ( fgets(buf,sizeof(buf),fp) != NULL )
	    {
	      /* Break the line up at ':' into two parts. */
	      p1 = strtok(buf,":");
	      p2 = strtok(NULL,":");
	      /* Get rid of [TAB] in /proc/cpuinfo if it's there */
	      p1 = strtok(p1,"\t");
	      p2 = strtok(p2,"\t");
	      if (strcmp(p1,"bogomips") == 0)
		{
		  bogo = (float) atoi(p2);
		}
	    }
	  if (bogo == 0.0)
	    {
	      fprintf(stderr,"warning: No bogomips rating in /proc/cpuinfo.\n");
	      fprintf(stderr,"\tUsing %.2f *mhz.\n",*mhz);
	    }
	  else
	    {
	      diff = (bogo - *mhz)/bogo*100.0;
	      if ( fabs(diff) > 1.0 )
		{
		  fprintf(stderr,
			  "warning: %.2f%% difference between\n",diff);
		  fprintf(stderr,
			  "\tcomputed %.2f *mhz and bogomips %.2f *mhz\n",
			  *mhz,bogo);
		}
	    }
	}
    }

  if (*mhz < 50. || *mhz > 500.)
    {
      fprintf(stderr,"warning: Suspicious CPU speed %.2f *mhz\n",*mhz);
    }

  if (!strcmp(mach,"i586"))
      *events = 40;

  if (!strcmp(mach,"i686"))
      *events = 68;

  return(0);
}

