#include <stdio.h>
#include <fcntl.h>
#include <math.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/processor.h>
#include <sys/procset.h>
#include "perfmon.h"

/* Do all the work here */
void loop() {
}

main(int argc, char *argv[])
{
    int fd;
    int rc;
    int cpu = 0;
    char c;
    extern char *optarg;
    extern int optind;
    register unsigned long long a, b;
    register unsigned long x, y;
    hrtime_t start, end;
    unsigned long long tmp;

    /* Parse command line arguments */
    while ((c = getopt(argc, argv, "p:")) != EOF)
	switch(c) {
	    /* -p cpu : binds this process to the indicated CPU */
	    case 'p':
		cpu = atoi(optarg);
		break;

	    default:
		break;
	}

    /* Try to bind us to the given CPU */
    rc = processor_bind(P_PID, P_MYID, cpu, NULL);
    if (rc == -1) {
	perror("processor_bind()");
	exit(1);
    }

    /* Try to open a channel to the perfmon driver */
    fd = open("/dev/perfmon", O_RDONLY);
    if (fd == -1) {
	perror("open(/dev/perfmon)");
	exit(1);
    }

    /* Set PCR to accumulate cycle count and number of instructions */
    tmp = PCR_S0_CYCLE_CNT | PCR_S1_INSTR_CNT | PCR_USER_TRACE;
    rc = ioctl(fd, PERFMON_SETPCR, &tmp);
    if (rc < 0) {
	perror("ioctl(PERFMON_SETPCR)");
	exit(1);
    }

    /* Run Loop */
    printf("Running loop...");
    fflush(stdout);

    start = gethrtime();

    clr_pic();
    cpu_sync();
    a = get_pic();

    loop();  /* Call the function that does all the work */

    cpu_sync();
    b = get_pic();

    end = gethrtime();

    printf("Done.\n\n");

    /* Calculate cold CPI */
    x = extract_pic0(b) - extract_pic0(a);  /* Number of cycles */
    y = extract_pic1(b) - extract_pic1(a);  /* Number of instructions */
    printf("Wall clock time used:            %.1lf msec\n",
	(end - start) / (double)1000000);
    printf("Number of instructions executed: %lu\n", y);
    printf("Number of cycles taken:          %lu\n", x);
    printf("CPI for loop:                    %.4lf\n", (double)x/(double)y);
    printf("IPC for loop:                    %.4lf\n", (double)y/(double)x);
}
